#ifndef AVCODEC_AVCODEC_57_H
#define AVCODEC_AVCODEC_57_H

/******************************************************************************
 * Structures needed in AVCodec major version 57.
 *
 * Exact version: 57.24.102
 */

#if LIBAVCODEC_VERSION_MAJOR == 57

typedef AVCodec AVCodec57;
typedef AVCodecContext AVCodecContext57;
typedef AVFrame AVFrame57;
typedef AVPacket AVPacket57;
typedef AVPixelFormat AVPixelFormat57;
typedef AVPacketSideDataType AVPacketSideDataType57;
typedef AVCodecID AVCodecID57;

#else

#ifdef AVCODEC_AVCODEC_56_H

//
// Get types from lower version.
//

typedef AVPacketSideDataType56 AVPacketSideDataType57;
typedef AVPixelFormat56 AVPixelFormat57;

#else

#error "AVPacketSideDataType57 not defined."

#endif // #ifdef AVCODEC_AVCODEC_56_H
/*
 * AVCodecContext 57.
 */

struct AVCodecContext57;

/*
 * AVFrame 57.
 */

typedef struct AVFrame57 {
#define AV_NUM_DATA_POINTERS_57 8
    /**
     * pointer to the picture/channel planes.
     * This might be different from the first allocated byte
     *
     * Some decoders access areas outside 0,0 - width,height, please
     * see avcodec_align_dimensions2(). Some filters and swscale can read
     * up to 16 bytes beyond the planes, if these filters are to be used,
     * then 16 extra bytes must be allocated.
     */
    uint8_t *data[AV_NUM_DATA_POINTERS_57];

    /**
     * For video, size in bytes of each picture line.
     * For audio, size in bytes of each plane.
     *
     * For audio, only linesize[0] may be set. For planar audio, each channel
     * plane must be the same size.
     *
     * For video the linesizes should be multiples of the CPUs alignment
     * preference, this is 16 or 32 for modern desktop CPUs.
     * Some code requires such alignment other code can be slower without
     * correct alignment, for yet other it makes no difference.
     *
     * @note The linesize may be larger than the size of usable data -- there
     * may be extra padding present for performance reasons.
     */
    int linesize[AV_NUM_DATA_POINTERS_57];

    /**
     * pointers to the data planes/channels.
     *
     * For video, this should simply point to data[].
     *
     * For planar audio, each channel has a separate data pointer, and
     * linesize[0] contains the size of each channel buffer.
     * For packed audio, there is just one data pointer, and linesize[0]
     * contains the total size of the buffer for all channels.
     *
     * Note: Both data and extended_data should always be set in a valid frame,
     * but for planar audio with more channels that can fit in data,
     * extended_data must be used in order to access all channels.
     */
    uint8_t **extended_data;

    /**
     * width and height of the video frame
     */
    int width, height;

    /**
     * number of audio samples (per channel) described by this frame
     */
    int nb_samples;

    /**
     * format of the frame, -1 if unknown or unset
     * Values correspond to enum AVPixelFormat for video frames,
     * enum AVSampleFormat for audio)
     */
    int format;

    /**
     * 1 -> keyframe, 0-> not
     */
    int key_frame;

    /**
     * Picture type of the frame.
     */
    enum AVPictureType pict_type;

    /**
     * Sample aspect ratio for the video frame, 0/1 if unknown/unspecified.
     */
    AVRational sample_aspect_ratio;

    /**
     * Presentation timestamp in time_base units (time when frame should be shown to user).
     */
    int64_t pts;

    /**
     * PTS copied from the AVPacket that was decoded to produce this frame.
     */
    int64_t pkt_pts;

    /**
     * DTS copied from the AVPacket that triggered returning this frame. (if frame threading isn't used)
     * This is also the Presentation time of this AVFrame calculated from
     * only AVPacket.dts values without pts values.
     */
    int64_t pkt_dts;

    /**
     * picture number in bitstream order
     */
    int coded_picture_number;
    /**
     * picture number in display order
     */
    int display_picture_number;

    /**
     * quality (between 1 (good) and FF_LAMBDA_MAX (bad))
     */
    int quality;

    /**
     * for some private data of the user
     */
    void *opaque;

#if FF_API_ERROR_FRAME
    /**
     * @deprecated unused
     */
    attribute_deprecated
    uint64_t error[AV_NUM_DATA_POINTERS_57];
#endif

    /**
     * When decoding, this signals how much the picture must be delayed.
     * extra_delay = repeat_pict / (2*fps)
     */
    int repeat_pict;

    /**
     * The content of the picture is interlaced.
     */
    int interlaced_frame;

    /**
     * If the content is interlaced, is top field displayed first.
     */
    int top_field_first;

    /**
     * Tell user application that palette has changed from previous frame.
     */
    int palette_has_changed;

    /**
     * reordered opaque 64bit (generally an integer or a double precision float
     * PTS but can be anything).
     * The user sets AVCodecContext.reordered_opaque to represent the input at
     * that time,
     * the decoder reorders values as needed and sets AVFrame.reordered_opaque
     * to exactly one of the values provided by the user through AVCodecContext.reordered_opaque
     * @deprecated in favor of pkt_pts
     */
    int64_t reordered_opaque;

    /**
     * Sample rate of the audio data.
     */
    int sample_rate;

    /**
     * Channel layout of the audio data.
     */
    uint64_t channel_layout;

    /**
     * AVBuffer references backing the data for this frame. If all elements of
     * this array are NULL, then this frame is not reference counted. This array
     * must be filled contiguously -- if buf[i] is non-NULL then buf[j] must
     * also be non-NULL for all j < i.
     *
     * There may be at most one AVBuffer per data plane, so for video this array
     * always contains all the references. For planar audio with more than
     * AV_NUM_DATA_POINTERS channels, there may be more buffers than can fit in
     * this array. Then the extra AVBufferRef pointers are stored in the
     * extended_buf array.
     */
    AVBufferRef *buf[AV_NUM_DATA_POINTERS_57];

    /**
     * For planar audio which requires more than AV_NUM_DATA_POINTERS
     * AVBufferRef pointers, this array will hold all the references which
     * cannot fit into AVFrame.buf.
     *
     * Note that this is different from AVFrame.extended_data, which always
     * contains all the pointers. This array only contains the extra pointers,
     * which cannot fit into AVFrame.buf.
     *
     * This array is always allocated using av_malloc() by whoever constructs
     * the frame. It is freed in av_frame_unref().
     */
    AVBufferRef **extended_buf;
    /**
     * Number of elements in extended_buf.
     */
    int        nb_extended_buf;

    AVFrameSideData **side_data;
    int            nb_side_data;

/**
 * @defgroup lavu_frame_flags AV_FRAME_FLAGS
 * Flags describing additional frame properties.
 *
 * @{
 */

/**
 * The frame data may be corrupted, e.g. due to decoding errors.
 */
#define AV_FRAME_FLAG_CORRUPT       (1 << 0)
/**
 * @}
 */

    /**
     * Frame flags, a combination of @ref lavu_frame_flags
     */
    int flags;

    /**
     * MPEG vs JPEG YUV range.
     * It must be accessed using av_frame_get_color_range() and
     * av_frame_set_color_range().
     * - encoding: Set by user
     * - decoding: Set by libavcodec
     */
    enum AVColorRange color_range;

    enum AVColorPrimaries color_primaries;

    enum AVColorTransferCharacteristic color_trc;

    /**
     * YUV colorspace type.
     * It must be accessed using av_frame_get_colorspace() and
     * av_frame_set_colorspace().
     * - encoding: Set by user
     * - decoding: Set by libavcodec
     */
    enum AVColorSpace colorspace;

    enum AVChromaLocation chroma_location;

    /**
     * frame timestamp estimated using various heuristics, in stream time base
     * Code outside libavutil should access this field using:
     * av_frame_get_best_effort_timestamp(frame)
     * - encoding: unused
     * - decoding: set by libavcodec, read by user.
     */
    int64_t best_effort_timestamp;

    /**
     * reordered pos from the last AVPacket that has been input into the decoder
     * Code outside libavutil should access this field using:
     * av_frame_get_pkt_pos(frame)
     * - encoding: unused
     * - decoding: Read by user.
     */
    int64_t pkt_pos;

    /**
     * duration of the corresponding packet, expressed in
     * AVStream->time_base units, 0 if unknown.
     * Code outside libavutil should access this field using:
     * av_frame_get_pkt_duration(frame)
     * - encoding: unused
     * - decoding: Read by user.
     */
    int64_t pkt_duration;

    /**
     * metadata.
     * Code outside libavutil should access this field using:
     * av_frame_get_metadata(frame)
     * - encoding: Set by user.
     * - decoding: Set by libavcodec.
     */
    AVDictionary *metadata;

    /**
     * decode error flags of the frame, set to a combination of
     * FF_DECODE_ERROR_xxx flags if the decoder produced a frame, but there
     * were errors during the decoding.
     * Code outside libavutil should access this field using:
     * av_frame_get_decode_error_flags(frame)
     * - encoding: unused
     * - decoding: set by libavcodec, read by user.
     */
    int decode_error_flags;
#define FF_DECODE_ERROR_INVALID_BITSTREAM_57   1
#define FF_DECODE_ERROR_MISSING_REFERENCE_57   2

    /**
     * number of audio channels, only used for audio.
     * Code outside libavutil should access this field using:
     * av_frame_get_channels(frame)
     * - encoding: unused
     * - decoding: Read by user.
     */
    int channels;

    /**
     * size of the corresponding packet containing the compressed
     * frame. It must be accessed using av_frame_get_pkt_size() and
     * av_frame_set_pkt_size().
     * It is set to a negative value if unknown.
     * - encoding: unused
     * - decoding: set by libavcodec, read by user.
     */
    int pkt_size;

#if FF_API_FRAME_QP
    /**
     * QP table
     * Not to be accessed directly from outside libavutil
     */
    attribute_deprecated
    int8_t *qscale_table;
    /**
     * QP store stride
     * Not to be accessed directly from outside libavutil
     */
    attribute_deprecated
    int qstride;

    attribute_deprecated
    int qscale_type;

    /**
     * Not to be accessed directly from outside libavutil
     */
    AVBufferRef *qp_table_buf;
#endif
} AVFrame57;

typedef struct AVPacketSideData57 {
    uint8_t *data;
    int      size;
    AVPacketSideDataType57 type;
} AVPacketSideData57;

/*
 * AVPacket 57.
 */

typedef struct AVPacket57 {
    /**
     * A reference to the reference-counted buffer where the packet data is
     * stored.
     * May be NULL, then the packet data is not reference-counted.
     */
    AVBufferRef *buf;
    /**
     * Presentation timestamp in AVStream->time_base units; the time at which
     * the decompressed packet will be presented to the user.
     * Can be AV_NOPTS_VALUE if it is not stored in the file.
     * pts MUST be larger or equal to dts as presentation cannot happen before
     * decompression, unless one wants to view hex dumps. Some formats misuse
     * the terms dts and pts/cts to mean something different. Such timestamps
     * must be converted to true pts/dts before they are stored in AVPacket.
     */
    int64_t pts;
    /**
     * Decompression timestamp in AVStream->time_base units; the time at which
     * the packet is decompressed.
     * Can be AV_NOPTS_VALUE if it is not stored in the file.
     */
    int64_t dts;
    uint8_t *data;
    int   size;
    int   stream_index;
    /**
     * A combination of AV_PKT_FLAG values
     */
    int   flags;
    /**
     * Additional packet data that can be provided by the container.
     * Packet can contain several types of side information.
     */
    AVPacketSideData57 *side_data;
    int side_data_elems;

    /**
     * Duration of this packet in AVStream->time_base units, 0 if unknown.
     * Equals next_pts - this_pts in presentation order.
     */
    int64_t duration;

    int64_t pos;                            ///< byte position in stream, -1 if unknown

#if FF_API_CONVERGENCE_DURATION
    /**
     * @deprecated Same as the duration field, but as int64_t. This was required
     * for Matroska subtitles, whose duration values could overflow when the
     * duration field was still an int.
     */
    attribute_deprecated
    int64_t convergence_duration;
#endif
} AVPacket57;

/**
 * Identify the syntax and semantics of the bitstream.
 * The principle is roughly:
 * Two decoders with the same ID can decode the same streams.
 * Two encoders with the same ID can encode compatible streams.
 * There may be slight deviations from the principle due to implementation
 * details.
 *
 * If you add a codec ID to this list, add it so that
 * 1. no value of a existing codec ID changes (that would break ABI),
 * 2. Give it a value which when taken as ASCII is recognized uniquely by a human as this specific codec.
 *    This ensures that 2 forks can independently add AVCodecIDs without producing conflicts.
 *
 * After adding new codec IDs, do not forget to add an entry to the codec
 * descriptor list and bump libavcodec minor version.
 */


enum AVCodecID57 {
    AV_CODEC_ID_NONE_57,

    /* video codecs */
    AV_CODEC_ID_MPEG1VIDEO_57,
    AV_CODEC_ID_MPEG2VIDEO_57, ///< preferred ID for MPEG-1/2 video decoding
#if FF_API_XVMC
    AV_CODEC_ID_MPEG2VIDEO_XVMC_57,
#endif /* FF_API_XVMC */
    AV_CODEC_ID_H261_57,
    AV_CODEC_ID_H263_57,
    AV_CODEC_ID_RV10_57,
    AV_CODEC_ID_RV20_57,
    AV_CODEC_ID_MJPEG_57,
    AV_CODEC_ID_MJPEGB_57,
    AV_CODEC_ID_LJPEG_57,
    AV_CODEC_ID_SP5X_57,
    AV_CODEC_ID_JPEGLS_57,
    AV_CODEC_ID_MPEG4_57,
    AV_CODEC_ID_RAWVIDEO_57,
    AV_CODEC_ID_MSMPEG4V1_57,
    AV_CODEC_ID_MSMPEG4V2_57,
    AV_CODEC_ID_MSMPEG4V3_57,
    AV_CODEC_ID_WMV1_57,
    AV_CODEC_ID_WMV2_57,
    AV_CODEC_ID_H263P_57,
    AV_CODEC_ID_H263I_57,
    AV_CODEC_ID_FLV1_57,
    AV_CODEC_ID_SVQ1_57,
    AV_CODEC_ID_SVQ3_57,
    AV_CODEC_ID_DVVIDEO_57,
    AV_CODEC_ID_HUFFYUV_57,
    AV_CODEC_ID_CYUV_57,
    AV_CODEC_ID_H264_57,
    AV_CODEC_ID_INDEO3_57,
    AV_CODEC_ID_VP3_57,
    AV_CODEC_ID_THEORA_57,
    AV_CODEC_ID_ASV1_57,
    AV_CODEC_ID_ASV2_57,
    AV_CODEC_ID_FFV1_57,
    AV_CODEC_ID_4XM_57,
    AV_CODEC_ID_VCR1_57,
    AV_CODEC_ID_CLJR_57,
    AV_CODEC_ID_MDEC_57,
    AV_CODEC_ID_ROQ_57,
    AV_CODEC_ID_INTERPLAY_VIDEO_57,
    AV_CODEC_ID_XAN_WC3_57,
    AV_CODEC_ID_XAN_WC4_57,
    AV_CODEC_ID_RPZA_57,
    AV_CODEC_ID_CINEPAK_57,
    AV_CODEC_ID_WS_VQA_57,
    AV_CODEC_ID_MSRLE_57,
    AV_CODEC_ID_MSVIDEO1_57,
    AV_CODEC_ID_IDCIN_57,
    AV_CODEC_ID_8BPS_57,
    AV_CODEC_ID_SMC_57,
    AV_CODEC_ID_FLIC_57,
    AV_CODEC_ID_TRUEMOTION1_57,
    AV_CODEC_ID_VMDVIDEO_57,
    AV_CODEC_ID_MSZH_57,
    AV_CODEC_ID_ZLIB_57,
    AV_CODEC_ID_QTRLE_57,
    AV_CODEC_ID_TSCC_57,
    AV_CODEC_ID_ULTI_57,
    AV_CODEC_ID_QDRAW_57,
    AV_CODEC_ID_VIXL_57,
    AV_CODEC_ID_QPEG_57,
    AV_CODEC_ID_PNG_57,
    AV_CODEC_ID_PPM_57,
    AV_CODEC_ID_PBM_57,
    AV_CODEC_ID_PGM_57,
    AV_CODEC_ID_PGMYUV_57,
    AV_CODEC_ID_PAM_57,
    AV_CODEC_ID_FFVHUFF_57,
    AV_CODEC_ID_RV30_57,
    AV_CODEC_ID_RV40_57,
    AV_CODEC_ID_VC1_57,
    AV_CODEC_ID_WMV3_57,
    AV_CODEC_ID_LOCO_57,
    AV_CODEC_ID_WNV1_57,
    AV_CODEC_ID_AASC_57,
    AV_CODEC_ID_INDEO2_57,
    AV_CODEC_ID_FRAPS_57,
    AV_CODEC_ID_TRUEMOTION2_57,
    AV_CODEC_ID_BMP_57,
    AV_CODEC_ID_CSCD_57,
    AV_CODEC_ID_MMVIDEO_57,
    AV_CODEC_ID_ZMBV_57,
    AV_CODEC_ID_AVS_57,
    AV_CODEC_ID_SMACKVIDEO_57,
    AV_CODEC_ID_NUV_57,
    AV_CODEC_ID_KMVC_57,
    AV_CODEC_ID_FLASHSV_57,
    AV_CODEC_ID_CAVS_57,
    AV_CODEC_ID_JPEG2000_57,
    AV_CODEC_ID_VMNC_57,
    AV_CODEC_ID_VP5_57,
    AV_CODEC_ID_VP6_57,
    AV_CODEC_ID_VP6F_57,
    AV_CODEC_ID_TARGA_57,
    AV_CODEC_ID_DSICINVIDEO_57,
    AV_CODEC_ID_TIERTEXSEQVIDEO_57,
    AV_CODEC_ID_TIFF_57,
    AV_CODEC_ID_GIF_57,
    AV_CODEC_ID_DXA_57,
    AV_CODEC_ID_DNXHD_57,
    AV_CODEC_ID_THP_57,
    AV_CODEC_ID_SGI_57,
    AV_CODEC_ID_C93_57,
    AV_CODEC_ID_BETHSOFTVID_57,
    AV_CODEC_ID_PTX_57,
    AV_CODEC_ID_TXD_57,
    AV_CODEC_ID_VP6A_57,
    AV_CODEC_ID_AMV_57,
    AV_CODEC_ID_VB_57,
    AV_CODEC_ID_PCX_57,
    AV_CODEC_ID_SUNRAST_57,
    AV_CODEC_ID_INDEO4_57,
    AV_CODEC_ID_INDEO5_57,
    AV_CODEC_ID_MIMIC_57,
    AV_CODEC_ID_RL2_57,
    AV_CODEC_ID_ESCAPE124_57,
    AV_CODEC_ID_DIRAC_57,
    AV_CODEC_ID_BFI_57,
    AV_CODEC_ID_CMV_57,
    AV_CODEC_ID_MOTIONPIXELS_57,
    AV_CODEC_ID_TGV_57,
    AV_CODEC_ID_TGQ_57,
    AV_CODEC_ID_TQI_57,
    AV_CODEC_ID_AURA_57,
    AV_CODEC_ID_AURA2_57,
    AV_CODEC_ID_V210X_57,
    AV_CODEC_ID_TMV_57,
    AV_CODEC_ID_V210_57,
    AV_CODEC_ID_DPX_57,
    AV_CODEC_ID_MAD_57,
    AV_CODEC_ID_FRWU_57,
    AV_CODEC_ID_FLASHSV2_57,
    AV_CODEC_ID_CDGRAPHICS_57,
    AV_CODEC_ID_R210_57,
    AV_CODEC_ID_ANM_57,
    AV_CODEC_ID_BINKVIDEO_57,
    AV_CODEC_ID_IFF_ILBM_57,
#define AV_CODEC_ID_IFF_BYTERUN1 AV_CODEC_ID_IFF_ILBM
    AV_CODEC_ID_KGV1_57,
    AV_CODEC_ID_YOP_57,
    AV_CODEC_ID_VP8_57,
    AV_CODEC_ID_PICTOR_57,
    AV_CODEC_ID_ANSI_57,
    AV_CODEC_ID_A64_MULTI_57,
    AV_CODEC_ID_A64_MULTI5_57,
    AV_CODEC_ID_R10K_57,
    AV_CODEC_ID_MXPEG_57,
    AV_CODEC_ID_LAGARITH_57,
    AV_CODEC_ID_PRORES_57,
    AV_CODEC_ID_JV_57,
    AV_CODEC_ID_DFA_57,
    AV_CODEC_ID_WMV3IMAGE_57,
    AV_CODEC_ID_VC1IMAGE_57,
    AV_CODEC_ID_UTVIDEO_57,
    AV_CODEC_ID_BMV_VIDEO_57,
    AV_CODEC_ID_VBLE_57,
    AV_CODEC_ID_DXTORY_57,
    AV_CODEC_ID_V410_57,
    AV_CODEC_ID_XWD_57,
    AV_CODEC_ID_CDXL_57,
    AV_CODEC_ID_XBM_57,
    AV_CODEC_ID_ZEROCODEC_57,
    AV_CODEC_ID_MSS1_57,
    AV_CODEC_ID_MSA1_57,
    AV_CODEC_ID_TSCC2_57,
    AV_CODEC_ID_MTS2_57,
    AV_CODEC_ID_CLLC_57,
    AV_CODEC_ID_MSS2_57,
    AV_CODEC_ID_VP9_57,
    AV_CODEC_ID_AIC_57,
    AV_CODEC_ID_ESCAPE130_57,
    AV_CODEC_ID_G2M_57,
    AV_CODEC_ID_WEBP_57,
    AV_CODEC_ID_HNM4_VIDEO_57,
    AV_CODEC_ID_HEVC_57,
#define AV_CODEC_ID_H265 AV_CODEC_ID_HEVC
    AV_CODEC_ID_FIC_57,
    AV_CODEC_ID_ALIAS_PIX_57,
    AV_CODEC_ID_BRENDER_PIX_57,
    AV_CODEC_ID_PAF_VIDEO_57,
    AV_CODEC_ID_EXR_57,
    AV_CODEC_ID_VP7_57,
    AV_CODEC_ID_SANM_57,
    AV_CODEC_ID_SGIRLE_57,
    AV_CODEC_ID_MVC1_57,
    AV_CODEC_ID_MVC2_57,
    AV_CODEC_ID_HQX_57,
    AV_CODEC_ID_TDSC_57,
    AV_CODEC_ID_HQ_HQA_57,
    AV_CODEC_ID_HAP_57,
    AV_CODEC_ID_DDS_57,
    AV_CODEC_ID_DXV_57,
    AV_CODEC_ID_SCREENPRESSO_57,
    AV_CODEC_ID_RSCC_57,

    AV_CODEC_ID_Y41P_57 = 0x8000,
    AV_CODEC_ID_AVRP_57,
    AV_CODEC_ID_012V_57,
    AV_CODEC_ID_AVUI_57,
    AV_CODEC_ID_AYUV_57,
    AV_CODEC_ID_TARGA_Y216_57,
    AV_CODEC_ID_V308_57,
    AV_CODEC_ID_V408_57,
    AV_CODEC_ID_YUV4_57,
    AV_CODEC_ID_AVRN_57,
    AV_CODEC_ID_CPIA_57,
    AV_CODEC_ID_XFACE_57,
    AV_CODEC_ID_SNOW_57,
    AV_CODEC_ID_SMVJPEG_57,
    AV_CODEC_ID_APNG_57,
    AV_CODEC_ID_DAALA_57,
    AV_CODEC_ID_CFHD_57,

    /* various PCM "codecs" */
    AV_CODEC_ID_FIRST_AUDIO_57 = 0x10000,     ///< A dummy id pointing at the start of audio codecs
    AV_CODEC_ID_PCM_S16LE_57 = 0x10000,
    AV_CODEC_ID_PCM_S16BE_57,
    AV_CODEC_ID_PCM_U16LE_57,
    AV_CODEC_ID_PCM_U16BE_57,
    AV_CODEC_ID_PCM_S8_57,
    AV_CODEC_ID_PCM_U8_57,
    AV_CODEC_ID_PCM_MULAW_57,
    AV_CODEC_ID_PCM_ALAW_57,
    AV_CODEC_ID_PCM_S32LE_57,
    AV_CODEC_ID_PCM_S32BE_57,
    AV_CODEC_ID_PCM_U32LE_57,
    AV_CODEC_ID_PCM_U32BE_57,
    AV_CODEC_ID_PCM_S24LE_57,
    AV_CODEC_ID_PCM_S24BE_57,
    AV_CODEC_ID_PCM_U24LE_57,
    AV_CODEC_ID_PCM_U24BE_57,
    AV_CODEC_ID_PCM_S24DAUD_57,
    AV_CODEC_ID_PCM_ZORK_57,
    AV_CODEC_ID_PCM_S16LE_PLANAR_57,
    AV_CODEC_ID_PCM_DVD_57,
    AV_CODEC_ID_PCM_F32BE_57,
    AV_CODEC_ID_PCM_F32LE_57,
    AV_CODEC_ID_PCM_F64BE_57,
    AV_CODEC_ID_PCM_F64LE_57,
    AV_CODEC_ID_PCM_BLURAY_57,
    AV_CODEC_ID_PCM_LXF_57,
    AV_CODEC_ID_S302M_57,
    AV_CODEC_ID_PCM_S8_PLANAR_57,
    AV_CODEC_ID_PCM_S24LE_PLANAR_57,
    AV_CODEC_ID_PCM_S32LE_PLANAR_57,
    AV_CODEC_ID_PCM_S16BE_PLANAR_57,
    /* new PCM "codecs" should be added right below this line starting with
     * an explicit value of for example 0x10800
     */

    /* various ADPCM codecs */
    AV_CODEC_ID_ADPCM_IMA_QT_57 = 0x11000,
    AV_CODEC_ID_ADPCM_IMA_WAV_57,
    AV_CODEC_ID_ADPCM_IMA_DK3_57,
    AV_CODEC_ID_ADPCM_IMA_DK4_57,
    AV_CODEC_ID_ADPCM_IMA_WS_57,
    AV_CODEC_ID_ADPCM_IMA_SMJPEG_57,
    AV_CODEC_ID_ADPCM_MS_57,
    AV_CODEC_ID_ADPCM_4XM_57,
    AV_CODEC_ID_ADPCM_XA_57,
    AV_CODEC_ID_ADPCM_ADX_57,
    AV_CODEC_ID_ADPCM_EA_57,
    AV_CODEC_ID_ADPCM_G726_57,
    AV_CODEC_ID_ADPCM_CT_57,
    AV_CODEC_ID_ADPCM_SWF_57,
    AV_CODEC_ID_ADPCM_YAMAHA_57,
    AV_CODEC_ID_ADPCM_SBPRO_4_57,
    AV_CODEC_ID_ADPCM_SBPRO_3_57,
    AV_CODEC_ID_ADPCM_SBPRO_2_57,
    AV_CODEC_ID_ADPCM_THP_57,
    AV_CODEC_ID_ADPCM_IMA_AMV_57,
    AV_CODEC_ID_ADPCM_EA_R1_57,
    AV_CODEC_ID_ADPCM_EA_R3_57,
    AV_CODEC_ID_ADPCM_EA_R2_57,
    AV_CODEC_ID_ADPCM_IMA_EA_SEAD_57,
    AV_CODEC_ID_ADPCM_IMA_EA_EACS_57,
    AV_CODEC_ID_ADPCM_EA_XAS_57,
    AV_CODEC_ID_ADPCM_EA_MAXIS_XA_57,
    AV_CODEC_ID_ADPCM_IMA_ISS_57,
    AV_CODEC_ID_ADPCM_G722_57,
    AV_CODEC_ID_ADPCM_IMA_APC_57,
    AV_CODEC_ID_ADPCM_VIMA_57,
#if FF_API_VIMA_DECODER
    AV_CODEC_ID_VIMA_57 = AV_CODEC_ID_ADPCM_VIMA_57,
#endif

    AV_CODEC_ID_ADPCM_AFC_57 = 0x11800,
    AV_CODEC_ID_ADPCM_IMA_OKI_57,
    AV_CODEC_ID_ADPCM_DTK_57,
    AV_CODEC_ID_ADPCM_IMA_RAD_57,
    AV_CODEC_ID_ADPCM_G726LE_57,
    AV_CODEC_ID_ADPCM_THP_LE_57,
    AV_CODEC_ID_ADPCM_PSX_57,
    AV_CODEC_ID_ADPCM_AICA_57,

    /* AMR */
    AV_CODEC_ID_AMR_NB_57 = 0x12000,
    AV_CODEC_ID_AMR_WB_57,

    /* RealAudio codecs*/
    AV_CODEC_ID_RA_144_57 = 0x13000,
    AV_CODEC_ID_RA_288_57,

    /* various DPCM codecs */
    AV_CODEC_ID_ROQ_DPCM_57 = 0x14000,
    AV_CODEC_ID_INTERPLAY_DPCM_57,
    AV_CODEC_ID_XAN_DPCM_57,
    AV_CODEC_ID_SOL_DPCM_57,

    AV_CODEC_ID_SDX2_DPCM_57 = 0x14800,

    /* audio codecs */
    AV_CODEC_ID_MP2_57 = 0x15000,
    AV_CODEC_ID_MP3_57, ///< preferred ID for decoding MPEG audio layer 1, 2 or 3
    AV_CODEC_ID_AAC_57,
    AV_CODEC_ID_AC3_57,
    AV_CODEC_ID_DTS_57,
    AV_CODEC_ID_VORBIS_57,
    AV_CODEC_ID_DVAUDIO_57,
    AV_CODEC_ID_WMAV1_57,
    AV_CODEC_ID_WMAV2_57,
    AV_CODEC_ID_MACE3_57,
    AV_CODEC_ID_MACE6_57,
    AV_CODEC_ID_VMDAUDIO_57,
    AV_CODEC_ID_FLAC_57,
    AV_CODEC_ID_MP3ADU_57,
    AV_CODEC_ID_MP3ON4_57,
    AV_CODEC_ID_SHORTEN_57,
    AV_CODEC_ID_ALAC_57,
    AV_CODEC_ID_WESTWOOD_SND1_57,
    AV_CODEC_ID_GSM_57, ///< as in Berlin toast format
    AV_CODEC_ID_QDM2_57,
    AV_CODEC_ID_COOK_57,
    AV_CODEC_ID_TRUESPEECH_57,
    AV_CODEC_ID_TTA_57,
    AV_CODEC_ID_SMACKAUDIO_57,
    AV_CODEC_ID_QCELP_57,
    AV_CODEC_ID_WAVPACK_57,
    AV_CODEC_ID_DSICINAUDIO_57,
    AV_CODEC_ID_IMC_57,
    AV_CODEC_ID_MUSEPACK7_57,
    AV_CODEC_ID_MLP_57,
    AV_CODEC_ID_GSM_MS_57, /* as found in WAV */
    AV_CODEC_ID_ATRAC3_57,
#if FF_API_VOXWARE
    AV_CODEC_ID_VOXWARE_57,
#endif
    AV_CODEC_ID_APE_57,
    AV_CODEC_ID_NELLYMOSER_57,
    AV_CODEC_ID_MUSEPACK8_57,
    AV_CODEC_ID_SPEEX_57,
    AV_CODEC_ID_WMAVOICE_57,
    AV_CODEC_ID_WMAPRO_57,
    AV_CODEC_ID_WMALOSSLESS_57,
    AV_CODEC_ID_ATRAC3P_57,
    AV_CODEC_ID_EAC3_57,
    AV_CODEC_ID_SIPR_57,
    AV_CODEC_ID_MP1_57,
    AV_CODEC_ID_TWINVQ_57,
    AV_CODEC_ID_TRUEHD_57,
    AV_CODEC_ID_MP4ALS_57,
    AV_CODEC_ID_ATRAC1_57,
    AV_CODEC_ID_BINKAUDIO_RDFT_57,
    AV_CODEC_ID_BINKAUDIO_DCT_57,
    AV_CODEC_ID_AAC_LATM_57,
    AV_CODEC_ID_QDMC_57,
    AV_CODEC_ID_CELT_57,
    AV_CODEC_ID_G723_1_57,
    AV_CODEC_ID_G729_57,
    AV_CODEC_ID_8SVX_EXP_57,
    AV_CODEC_ID_8SVX_FIB_57,
    AV_CODEC_ID_BMV_AUDIO_57,
    AV_CODEC_ID_RALF_57,
    AV_CODEC_ID_IAC_57,
    AV_CODEC_ID_ILBC_57,
    AV_CODEC_ID_OPUS_57,
    AV_CODEC_ID_COMFORT_NOISE_57,
    AV_CODEC_ID_TAK_57,
    AV_CODEC_ID_METASOUND_57,
    AV_CODEC_ID_PAF_AUDIO_57,
    AV_CODEC_ID_ON2AVC_57,
    AV_CODEC_ID_DSS_SP_57,

    AV_CODEC_ID_FFWAVESYNTH_57 = 0x15800,
    AV_CODEC_ID_SONIC_57,
    AV_CODEC_ID_SONIC_LS_57,
    AV_CODEC_ID_EVRC_57,
    AV_CODEC_ID_SMV_57,
    AV_CODEC_ID_DSD_LSBF_57,
    AV_CODEC_ID_DSD_MSBF_57,
    AV_CODEC_ID_DSD_LSBF_PLANAR_57,
    AV_CODEC_ID_DSD_MSBF_PLANAR_57,
    AV_CODEC_ID_4GV_57,
    AV_CODEC_ID_INTERPLAY_ACM_57,
    AV_CODEC_ID_XMA1_57,
    AV_CODEC_ID_XMA2_57,

    /* subtitle codecs */
    AV_CODEC_ID_FIRST_SUBTITLE_57 = 0x17000,          ///< A dummy ID pointing at the start of subtitle codecs.
    AV_CODEC_ID_DVD_SUBTITLE_57 = 0x17000,
    AV_CODEC_ID_DVB_SUBTITLE_57,
    AV_CODEC_ID_TEXT_57,  ///< raw UTF-8 text
    AV_CODEC_ID_XSUB_57,
    AV_CODEC_ID_SSA_57,
    AV_CODEC_ID_MOV_TEXT_57,
    AV_CODEC_ID_HDMV_PGS_SUBTITLE_57,
    AV_CODEC_ID_DVB_TELETEXT_57,
    AV_CODEC_ID_SRT_57,

    AV_CODEC_ID_MICRODVD_57   = 0x17800,
    AV_CODEC_ID_EIA_608_57,
    AV_CODEC_ID_JACOSUB_57,
    AV_CODEC_ID_SAMI_57,
    AV_CODEC_ID_REALTEXT_57,
    AV_CODEC_ID_STL_57,
    AV_CODEC_ID_SUBVIEWER1_57,
    AV_CODEC_ID_SUBVIEWER_57,
    AV_CODEC_ID_SUBRIP_57,
    AV_CODEC_ID_WEBVTT_57,
    AV_CODEC_ID_MPL2_57,
    AV_CODEC_ID_VPLAYER_57,
    AV_CODEC_ID_PJS_57,
    AV_CODEC_ID_ASS_57,
    AV_CODEC_ID_HDMV_TEXT_SUBTITLE_57,

    /* other specific kind of codecs (generally used for attachments) */
    AV_CODEC_ID_FIRST_UNKNOWN_57 = 0x18000,           ///< A dummy ID pointing at the start of various fake codecs.
    AV_CODEC_ID_TTF_57 = 0x18000,

    AV_CODEC_ID_BINTEXT_57    = 0x18800,
    AV_CODEC_ID_XBIN_57,
    AV_CODEC_ID_IDF_57,
    AV_CODEC_ID_OTF_57,
    AV_CODEC_ID_SMPTE_KLV_57,
    AV_CODEC_ID_DVD_NAV_57,
    AV_CODEC_ID_TIMED_ID3_57,
    AV_CODEC_ID_BIN_DATA_57,


    AV_CODEC_ID_PROBE_57 = 0x19000, ///< codec_id is not known (like AV_CODEC_ID_NONE) but lavf should attempt to identify it

    AV_CODEC_ID_MPEG2TS_57 = 0x20000, /**< _FAKE_ codec to indicate a raw MPEG-2 TS
                                * stream (only used by libavformat) */
    AV_CODEC_ID_MPEG4SYSTEMS_57 = 0x20001, /**< _FAKE_ codec to indicate a MPEG-4 Systems
                                * stream (only used by libavformat) */
    AV_CODEC_ID_FFMETADATA_57 = 0x21000,   ///< Dummy codec for streams containing only metadata information.
    AV_CODEC_ID_WRAPPED_AVFRAME_57 = 0x21001, ///< Passthrough codec, AVFrames wrapped in AVPacket
};

typedef enum AVCodecID57 AVCodecID57;


/**
 * AVCodec 57.
 */

typedef struct AVCodec57 {
    /**
     * Name of the codec implementation.
     * The name is globally unique among encoders and among decoders (but an
     * encoder and a decoder can share the same name).
     * This is the primary way to find a codec from the user perspective.
     */
    const char *name;
    /**
     * Descriptive name for the codec, meant to be more human readable than name.
     * You should use the NULL_IF_CONFIG_SMALL() macro to define it.
     */
    const char *long_name;
    enum AVMediaType type;
    enum AVCodecID57 id;
    /**
     * Codec capabilities.
     * see AV_CODEC_CAP_*
     */
    int capabilities;
    const AVRational *supported_framerates; ///< array of supported framerates, or NULL if any, array is terminated by {0,0}
    const AVPixelFormat57 *pix_fmts;     ///< array of supported pixel formats, or NULL if unknown, array is terminated by -1
    const int *supported_samplerates;       ///< array of supported audio samplerates, or NULL if unknown, array is terminated by 0
    const enum AVSampleFormat *sample_fmts; ///< array of supported sample formats, or NULL if unknown, array is terminated by -1
    const uint64_t *channel_layouts;         ///< array of support channel layouts, or NULL if unknown. array is terminated by 0
    uint8_t max_lowres;                     ///< maximum value for lowres supported by the decoder, no direct access, use av_codec_get_max_lowres()
    const AVClass *priv_class;              ///< AVClass for the private context
    const AVProfile *profiles;              ///< array of recognized profiles, or NULL if unknown, array is terminated by {FF_PROFILE_UNKNOWN}

    /*****************************************************************
     * No fields below this line are part of the public API. They
     * may not be used outside of libavcodec and can be changed and
     * removed at will.
     * New public fields should be added right above.
     *****************************************************************
     */
    int priv_data_size;
    struct AVCodec57 *next;
    /**
     * @name Frame-level threading support functions
     * @{
     */
    /**
     * If defined, called on thread contexts when they are created.
     * If the codec allocates writable tables in init(), re-allocate them here.
     * priv_data will be set to a copy of the original.
     */
    int (*init_thread_copy)(struct AVCodecContext57 *);
    /**
     * Copy necessary context variables from a previous thread context to the current one.
     * If not defined, the next thread will start automatically; otherwise, the codec
     * must call ff_thread_finish_setup().
     *
     * dst and src will (rarely) point to the same context, in which case memcpy should be skipped.
     */
    int (*update_thread_context)(struct AVCodecContext57 *dst, const struct AVCodecContext57 *src);
    /** @} */

    /**
     * Private codec-specific defaults.
     */
    const AVCodecDefault *defaults;

    /**
     * Initialize codec static data, called from avcodec_register().
     */
    void (*init_static_data)(struct AVCodec57 *codec);

    int (*init)(struct AVCodecContext57 *);
    int (*encode_sub)(struct AVCodecContext57 *, uint8_t *buf, int buf_size,
                      const struct AVSubtitle *sub);
    /**
     * Encode data to an AVPacket.
     *
     * @param      avctx          codec context
     * @param      avpkt          output AVPacket (may contain a user-provided buffer)
     * @param[in]  frame          AVFrame containing the raw data to be encoded
     * @param[out] got_packet_ptr encoder sets to 0 or 1 to indicate that a
     *                            non-empty packet was returned in avpkt.
     * @return 0 on success, negative error code on failure
     */
    int (*encode2)(struct AVCodecContext57 *avctx, AVPacket57 *avpkt, const AVFrame57 *frame,
                   int *got_packet_ptr);
    int (*decode)(struct AVCodecContext57 *, void *outdata, int *outdata_size, AVPacket57 *avpkt);
    int (*close)(struct AVCodecContext57 *);
    /**
     * Flush buffers.
     * Will be called when seeking
     */
    void (*flush)(struct AVCodecContext57 *);
    /**
     * Internal codec capabilities.
     * See FF_CODEC_CAP_* in internal.h
     */
    int caps_internal;
} AVCodec57;

/*
 * AVCodecContext57.
 */

typedef struct AVCodecContext57 {
    /**
     * information on struct for av_log
     * - set by avcodec_alloc_context3
     */
    const AVClass *av_class;
    int log_level_offset;

    enum AVMediaType codec_type; /* see AVMEDIA_TYPE_xxx */
    const struct AVCodec57  *codec;
#if FF_API_CODEC_NAME
    /**
     * @deprecated this field is not used for anything in libavcodec
     */
    attribute_deprecated
    char             codec_name[32];
#endif
    enum AVCodecID57     codec_id; /* see AV_CODEC_ID_xxx */

    /**
     * fourcc (LSB first, so "ABCD" -> ('D'<<24) + ('C'<<16) + ('B'<<8) + 'A').
     * This is used to work around some encoder bugs.
     * A demuxer should set this to what is stored in the field used to identify the codec.
     * If there are multiple such fields in a container then the demuxer should choose the one
     * which maximizes the information about the used codec.
     * If the codec tag field in a container is larger than 32 bits then the demuxer should
     * remap the longer ID to 32 bits with a table or other structure. Alternatively a new
     * extra_codec_tag + size could be added but for this a clear advantage must be demonstrated
     * first.
     * - encoding: Set by user, if not then the default based on codec_id will be used.
     * - decoding: Set by user, will be converted to uppercase by libavcodec during init.
     */
    unsigned int codec_tag;

#if FF_API_STREAM_CODEC_TAG
    /**
     * @deprecated this field is unused
     */
    attribute_deprecated
    unsigned int stream_codec_tag;
#endif

    void *priv_data;

    /**
     * Private context used for internal data.
     *
     * Unlike priv_data, this is not codec-specific. It is used in general
     * libavcodec functions.
     */
    struct AVCodecInternal *internal;

    /**
     * Private data of the user, can be used to carry app specific stuff.
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    void *opaque;

    /**
     * the average bitrate
     * - encoding: Set by user; unused for constant quantizer encoding.
     * - decoding: Set by user, may be overwritten by libavcodec
     *             if this info is available in the stream
     */
    int64_t bit_rate;

    /**
     * number of bits the bitstream is allowed to diverge from the reference.
     *           the reference can be CBR (for CBR pass1) or VBR (for pass2)
     * - encoding: Set by user; unused for constant quantizer encoding.
     * - decoding: unused
     */
    int bit_rate_tolerance;

    /**
     * Global quality for codecs which cannot change it per frame.
     * This should be proportional to MPEG-1/2/4 qscale.
     * - encoding: Set by user.
     * - decoding: unused
     */
    int global_quality;

    /**
     * - encoding: Set by user.
     * - decoding: unused
     */
    int compression_level;

    /**
     * AV_CODEC_FLAG_*.
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    int flags;

    /**
     * AV_CODEC_FLAG2_*
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    int flags2;

    /**
     * some codecs need / can use extradata like Huffman tables.
     * mjpeg: Huffman tables
     * rv10: additional flags
     * mpeg4: global headers (they can be in the bitstream or here)
     * The allocated memory should be AV_INPUT_BUFFER_PADDING_SIZE bytes larger
     * than extradata_size to avoid problems if it is read with the bitstream reader.
     * The bytewise contents of extradata must not depend on the architecture or CPU endianness.
     * - encoding: Set/allocated/freed by libavcodec.
     * - decoding: Set/allocated/freed by user.
     */
    uint8_t *extradata;
    int extradata_size;

    /**
     * This is the fundamental unit of time (in seconds) in terms
     * of which frame timestamps are represented. For fixed-fps content,
     * timebase should be 1/framerate and timestamp increments should be
     * identically 1.
     * This often, but not always is the inverse of the frame rate or field rate
     * for video.
     * - encoding: MUST be set by user.
     * - decoding: the use of this field for decoding is deprecated.
     *             Use framerate instead.
     */
    AVRational time_base;

    /**
     * For some codecs, the time base is closer to the field rate than the frame rate.
     * Most notably, H.264 and MPEG-2 specify time_base as half of frame duration
     * if no telecine is used ...
     *
     * Set to time_base ticks per frame. Default 1, e.g., H.264/MPEG-2 set it to 2.
     */
    int ticks_per_frame;

    /**
     * Codec delay.
     *
     * Encoding: Number of frames delay there will be from the encoder input to
     *           the decoder output. (we assume the decoder matches the spec)
     * Decoding: Number of frames delay in addition to what a standard decoder
     *           as specified in the spec would produce.
     *
     * Video:
     *   Number of frames the decoded output will be delayed relative to the
     *   encoded input.
     *
     * Audio:
     *   For encoding, this field is unused (see initial_padding).
     *
     *   For decoding, this is the number of samples the decoder needs to
     *   output before the decoder's output is valid. When seeking, you should
     *   start decoding this many samples prior to your desired seek point.
     *
     * - encoding: Set by libavcodec.
     * - decoding: Set by libavcodec.
     */
    int delay;


    /* video only */
    /**
     * picture width / height.
     *
     * @note Those fields may not match the values of the last
     * AVFrame outputted by avcodec_decode_video2 due frame
     * reordering.
     *
     * - encoding: MUST be set by user.
     * - decoding: May be set by the user before opening the decoder if known e.g.
     *             from the container. Some decoders will require the dimensions
     *             to be set by the caller. During decoding, the decoder may
     *             overwrite those values as required while parsing the data.
     */
    int width, height;

    /**
     * Bitstream width / height, may be different from width/height e.g. when
     * the decoded frame is cropped before being output or lowres is enabled.
     *
     * @note Those field may not match the value of the last
     * AVFrame outputted by avcodec_decode_video2 due frame
     * reordering.
     *
     * - encoding: unused
     * - decoding: May be set by the user before opening the decoder if known
     *             e.g. from the container. During decoding, the decoder may
     *             overwrite those values as required while parsing the data.
     */
    int coded_width, coded_height;

#if FF_API_ASPECT_EXTENDED
#define FF_ASPECT_EXTENDED_57 15
#endif

    /**
     * the number of pictures in a group of pictures, or 0 for intra_only
     * - encoding: Set by user.
     * - decoding: unused
     */
    int gop_size;

    /**
     * Pixel format, see AV_PIX_FMT_xxx.
     * May be set by the demuxer if known from headers.
     * May be overridden by the decoder if it knows better.
     *
     * @note This field may not match the value of the last
     * AVFrame outputted by avcodec_decode_video2 due frame
     * reordering.
     *
     * - encoding: Set by user.
     * - decoding: Set by user if known, overridden by libavcodec while
     *             parsing the data.
     */
    AVPixelFormat57 pix_fmt;

#if FF_API_MOTION_EST
    /**
     * This option does nothing
     * @deprecated use codec private options instead
     */
    attribute_deprecated int me_method;
#endif

    /**
     * If non NULL, 'draw_horiz_band' is called by the libavcodec
     * decoder to draw a horizontal band. It improves cache usage. Not
     * all codecs can do that. You must check the codec capabilities
     * beforehand.
     * When multithreading is used, it may be called from multiple threads
     * at the same time; threads might draw different parts of the same AVFrame,
     * or multiple AVFrames, and there is no guarantee that slices will be drawn
     * in order.
     * The function is also used by hardware acceleration APIs.
     * It is called at least once during frame decoding to pass
     * the data needed for hardware render.
     * In that mode instead of pixel data, AVFrame points to
     * a structure specific to the acceleration API. The application
     * reads the structure and can change some fields to indicate progress
     * or mark state.
     * - encoding: unused
     * - decoding: Set by user.
     * @param height the height of the slice
     * @param y the y position of the slice
     * @param type 1->top field, 2->bottom field, 3->frame
     * @param offset offset into the AVFrame.data from which the slice should be read
     */
    void (*draw_horiz_band)(struct AVCodecContext57 *s,
                            const AVFrame57 *src, int offset[AV_NUM_DATA_POINTERS],
                            int y, int type, int height);

    /**
     * callback to negotiate the pixelFormat
     * @param fmt is the list of formats which are supported by the codec,
     * it is terminated by -1 as 0 is a valid format, the formats are ordered by quality.
     * The first is always the native one.
     * @note The callback may be called again immediately if initialization for
     * the selected (hardware-accelerated) pixel format failed.
     * @warning Behavior is undefined if the callback returns a value not
     * in the fmt list of formats.
     * @return the chosen format
     * - encoding: unused
     * - decoding: Set by user, if not set the native format will be chosen.
     */
    AVPixelFormat57 (*get_format)(struct AVCodecContext57 *s, const AVPixelFormat57 * fmt);

    /**
     * maximum number of B-frames between non-B-frames
     * Note: The output will be delayed by max_b_frames+1 relative to the input.
     * - encoding: Set by user.
     * - decoding: unused
     */
    int max_b_frames;

    /**
     * qscale factor between IP and B-frames
     * If > 0 then the last P-frame quantizer will be used (q= lastp_q*factor+offset).
     * If < 0 then normal ratecontrol will be done (q= -normal_q*factor+offset).
     * - encoding: Set by user.
     * - decoding: unused
     */
    float b_quant_factor;

#if FF_API_RC_STRATEGY
    /** @deprecated use codec private option instead */
    attribute_deprecated int rc_strategy;
#define FF_RC_STRATEGY_XVID_57 1
#endif

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int b_frame_strategy;
#endif

    /**
     * qscale offset between IP and B-frames
     * - encoding: Set by user.
     * - decoding: unused
     */
    float b_quant_offset;

    /**
     * Size of the frame reordering buffer in the decoder.
     * For MPEG-2 it is 1 IPB or 0 low delay IP.
     * - encoding: Set by libavcodec.
     * - decoding: Set by libavcodec.
     */
    int has_b_frames;

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int mpeg_quant;
#endif

    /**
     * qscale factor between P and I-frames
     * If > 0 then the last p frame quantizer will be used (q= lastp_q*factor+offset).
     * If < 0 then normal ratecontrol will be done (q= -normal_q*factor+offset).
     * - encoding: Set by user.
     * - decoding: unused
     */
    float i_quant_factor;

    /**
     * qscale offset between P and I-frames
     * - encoding: Set by user.
     * - decoding: unused
     */
    float i_quant_offset;

    /**
     * luminance masking (0-> disabled)
     * - encoding: Set by user.
     * - decoding: unused
     */
    float lumi_masking;

    /**
     * temporary complexity masking (0-> disabled)
     * - encoding: Set by user.
     * - decoding: unused
     */
    float temporal_cplx_masking;

    /**
     * spatial complexity masking (0-> disabled)
     * - encoding: Set by user.
     * - decoding: unused
     */
    float spatial_cplx_masking;

    /**
     * p block masking (0-> disabled)
     * - encoding: Set by user.
     * - decoding: unused
     */
    float p_masking;

    /**
     * darkness masking (0-> disabled)
     * - encoding: Set by user.
     * - decoding: unused
     */
    float dark_masking;

    /**
     * slice count
     * - encoding: Set by libavcodec.
     * - decoding: Set by user (or 0).
     */
    int slice_count;

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
     int prediction_method;
#define FF_PRED_LEFT_57   0
#define FF_PRED_PLANE_57  1
#define FF_PRED_MEDIAN_57 2
#endif

    /**
     * slice offsets in the frame in bytes
     * - encoding: Set/allocated by libavcodec.
     * - decoding: Set/allocated by user (or NULL).
     */
    int *slice_offset;

    /**
     * sample aspect ratio (0 if unknown)
     * That is the width of a pixel divided by the height of the pixel.
     * Numerator and denominator must be relatively prime and smaller than 256 for some video standards.
     * - encoding: Set by user.
     * - decoding: Set by libavcodec.
     */
    AVRational sample_aspect_ratio;

    /**
     * motion estimation comparison function
     * - encoding: Set by user.
     * - decoding: unused
     */
    int me_cmp;
    /**
     * subpixel motion estimation comparison function
     * - encoding: Set by user.
     * - decoding: unused
     */
    int me_sub_cmp;
    /**
     * macroblock comparison function (not supported yet)
     * - encoding: Set by user.
     * - decoding: unused
     */
    int mb_cmp;
    /**
     * interlaced DCT comparison function
     * - encoding: Set by user.
     * - decoding: unused
     */
    int ildct_cmp;
#define FF_CMP_SAD_57    0
#define FF_CMP_SSE_57    1
#define FF_CMP_SATD_57   2
#define FF_CMP_DCT_57    3
#define FF_CMP_PSNR_57   4
#define FF_CMP_BIT_57    5
#define FF_CMP_RD_57     6
#define FF_CMP_ZERO_57   7
#define FF_CMP_VSAD_57   8
#define FF_CMP_VSSE_57   9
#define FF_CMP_NSSE_57   10
#define FF_CMP_W53_57    11
#define FF_CMP_W97_57    12
#define FF_CMP_DCTMAX_57 13
#define FF_CMP_DCT264_57 14
#define FF_CMP_CHROMA_57 256

    /**
     * ME diamond size & shape
     * - encoding: Set by user.
     * - decoding: unused
     */
    int dia_size;

    /**
     * amount of previous MV predictors (2a+1 x 2a+1 square)
     * - encoding: Set by user.
     * - decoding: unused
     */
    int last_predictor_count;

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int pre_me;
#endif

    /**
     * motion estimation prepass comparison function
     * - encoding: Set by user.
     * - decoding: unused
     */
    int me_pre_cmp;

    /**
     * ME prepass diamond size & shape
     * - encoding: Set by user.
     * - decoding: unused
     */
    int pre_dia_size;

    /**
     * subpel ME quality
     * - encoding: Set by user.
     * - decoding: unused
     */
    int me_subpel_quality;

#if FF_API_AFD
    /**
     * DTG active format information (additional aspect ratio
     * information only used in DVB MPEG-2 transport streams)
     * 0 if not set.
     *
     * - encoding: unused
     * - decoding: Set by decoder.
     * @deprecated Deprecated in favor of AVSideData
     */
    attribute_deprecated int dtg_active_format;
#define FF_DTG_AFD_SAME_57         8
#define FF_DTG_AFD_4_3_57          9
#define FF_DTG_AFD_16_9_57         10
#define FF_DTG_AFD_14_9_57         11
#define FF_DTG_AFD_4_3_SP_14_9_57  13
#define FF_DTG_AFD_16_9_SP_14_9_57 14
#define FF_DTG_AFD_SP_4_3_57       15
#endif /* FF_API_AFD */

    /**
     * maximum motion estimation search range in subpel units
     * If 0 then no limit.
     *
     * - encoding: Set by user.
     * - decoding: unused
     */
    int me_range;

#if FF_API_QUANT_BIAS
    /**
     * @deprecated use encoder private option instead
     */
    attribute_deprecated int intra_quant_bias;
#define FF_DEFAULT_QUANT_BIAS 999999

    /**
     * @deprecated use encoder private option instead
     */
    attribute_deprecated int inter_quant_bias;
#endif

    /**
     * slice flags
     * - encoding: unused
     * - decoding: Set by user.
     */
    int slice_flags;
#define SLICE_FLAG_CODED_ORDER_57    0x0001 ///< draw_horiz_band() is called in coded order instead of display
#define SLICE_FLAG_ALLOW_FIELD_57    0x0002 ///< allow draw_horiz_band() with field slices (MPEG2 field pics)
#define SLICE_FLAG_ALLOW_PLANE_57    0x0004 ///< allow draw_horiz_band() with 1 component at a time (SVQ1)

#if FF_API_XVMC
    /**
     * XVideo Motion Acceleration
     * - encoding: forbidden
     * - decoding: set by decoder
     * @deprecated XvMC doesn't need it anymore.
     */
    attribute_deprecated int xvmc_acceleration;
#endif /* FF_API_XVMC */

    /**
     * macroblock decision mode
     * - encoding: Set by user.
     * - decoding: unused
     */
    int mb_decision;
#define FF_MB_DECISION_SIMPLE_57 0        ///< uses mb_cmp
#define FF_MB_DECISION_BITS_57   1        ///< chooses the one which needs the fewest bits
#define FF_MB_DECISION_RD_57     2        ///< rate distortion

    /**
     * custom intra quantization matrix
     * - encoding: Set by user, can be NULL.
     * - decoding: Set by libavcodec.
     */
    uint16_t *intra_matrix;

    /**
     * custom inter quantization matrix
     * - encoding: Set by user, can be NULL.
     * - decoding: Set by libavcodec.
     */
    uint16_t *inter_matrix;

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int scenechange_threshold;

    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int noise_reduction;
#endif

#if FF_API_MPV_OPT
    /**
     * @deprecated this field is unused
     */
    attribute_deprecated
    int me_threshold;

    /**
     * @deprecated this field is unused
     */
    attribute_deprecated
    int mb_threshold;
#endif

    /**
     * precision of the intra DC coefficient - 8
     * - encoding: Set by user.
     * - decoding: Set by libavcodec
     */
    int intra_dc_precision;

    /**
     * Number of macroblock rows at the top which are skipped.
     * - encoding: unused
     * - decoding: Set by user.
     */
    int skip_top;

    /**
     * Number of macroblock rows at the bottom which are skipped.
     * - encoding: unused
     * - decoding: Set by user.
     */
    int skip_bottom;

#if FF_API_MPV_OPT
    /**
     * @deprecated use encoder private options instead
     */
    attribute_deprecated
    float border_masking;
#endif

    /**
     * minimum MB lagrange multipler
     * - encoding: Set by user.
     * - decoding: unused
     */
    int mb_lmin;

    /**
     * maximum MB lagrange multipler
     * - encoding: Set by user.
     * - decoding: unused
     */
    int mb_lmax;

#if FF_API_PRIVATE_OPT
    /**
     * @deprecated use encoder private options instead
     */
    attribute_deprecated
    int me_penalty_compensation;
#endif

    /**
     *
     * - encoding: Set by user.
     * - decoding: unused
     */
    int bidir_refine;

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int brd_scale;
#endif

    /**
     * minimum GOP size
     * - encoding: Set by user.
     * - decoding: unused
     */
    int keyint_min;

    /**
     * number of reference frames
     * - encoding: Set by user.
     * - decoding: Set by lavc.
     */
    int refs;

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int chromaoffset;
#endif

#if FF_API_UNUSED_MEMBERS
    /**
     * Multiplied by qscale for each frame and added to scene_change_score.
     * - encoding: Set by user.
     * - decoding: unused
     */
    attribute_deprecated int scenechange_factor;
#endif

    /**
     *
     * Note: Value depends upon the compare function used for fullpel ME.
     * - encoding: Set by user.
     * - decoding: unused
     */
    int mv0_threshold;

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int b_sensitivity;
#endif

    /**
     * Chromaticity coordinates of the source primaries.
     * - encoding: Set by user
     * - decoding: Set by libavcodec
     */
    enum AVColorPrimaries color_primaries;

    /**
     * Color Transfer Characteristic.
     * - encoding: Set by user
     * - decoding: Set by libavcodec
     */
    enum AVColorTransferCharacteristic color_trc;

    /**
     * YUV colorspace type.
     * - encoding: Set by user
     * - decoding: Set by libavcodec
     */
    enum AVColorSpace colorspace;

    /**
     * MPEG vs JPEG YUV range.
     * - encoding: Set by user
     * - decoding: Set by libavcodec
     */
    enum AVColorRange color_range;

    /**
     * This defines the location of chroma samples.
     * - encoding: Set by user
     * - decoding: Set by libavcodec
     */
    enum AVChromaLocation chroma_sample_location;

    /**
     * Number of slices.
     * Indicates number of picture subdivisions. Used for parallelized
     * decoding.
     * - encoding: Set by user
     * - decoding: unused
     */
    int slices;

    /** Field order
     * - encoding: set by libavcodec
     * - decoding: Set by user.
     */
    enum AVFieldOrder field_order;

    /* audio only */
    int sample_rate; ///< samples per second
    int channels;    ///< number of audio channels

    /**
     * audio sample format
     * - encoding: Set by user.
     * - decoding: Set by libavcodec.
     */
    enum AVSampleFormat sample_fmt;  ///< sample format

    /* The following data should not be initialized. */
    /**
     * Number of samples per channel in an audio frame.
     *
     * - encoding: set by libavcodec in avcodec_open2(). Each submitted frame
     *   except the last must contain exactly frame_size samples per channel.
     *   May be 0 when the codec has AV_CODEC_CAP_VARIABLE_FRAME_SIZE set, then the
     *   frame size is not restricted.
     * - decoding: may be set by some decoders to indicate constant frame size
     */
    int frame_size;

    /**
     * Frame counter, set by libavcodec.
     *
     * - decoding: total number of frames returned from the decoder so far.
     * - encoding: total number of frames passed to the encoder so far.
     *
     *   @note the counter is not incremented if encoding/decoding resulted in
     *   an error.
     */
    int frame_number;

    /**
     * number of bytes per packet if constant and known or 0
     * Used by some WAV based audio codecs.
     */
    int block_align;

    /**
     * Audio cutoff bandwidth (0 means "automatic")
     * - encoding: Set by user.
     * - decoding: unused
     */
    int cutoff;

    /**
     * Audio channel layout.
     * - encoding: set by user.
     * - decoding: set by user, may be overwritten by libavcodec.
     */
    uint64_t channel_layout;

    /**
     * Request decoder to use this channel layout if it can (0 for default)
     * - encoding: unused
     * - decoding: Set by user.
     */
    uint64_t request_channel_layout;

    /**
     * Type of service that the audio stream conveys.
     * - encoding: Set by user.
     * - decoding: Set by libavcodec.
     */
    enum AVAudioServiceType audio_service_type;

    /**
     * desired sample format
     * - encoding: Not used.
     * - decoding: Set by user.
     * Decoder will decode to this format if it can.
     */
    enum AVSampleFormat request_sample_fmt;

    /**
     * This callback is called at the beginning of each frame to get data
     * buffer(s) for it. There may be one contiguous buffer for all the data or
     * there may be a buffer per each data plane or anything in between. What
     * this means is, you may set however many entries in buf[] you feel necessary.
     * Each buffer must be reference-counted using the AVBuffer API (see description
     * of buf[] below).
     *
     * The following fields will be set in the frame before this callback is
     * called:
     * - format
     * - width, height (video only)
     * - sample_rate, channel_layout, nb_samples (audio only)
     * Their values may differ from the corresponding values in
     * AVCodecContext. This callback must use the frame values, not the codec
     * context values, to calculate the required buffer size.
     *
     * This callback must fill the following fields in the frame:
     * - data[]
     * - linesize[]
     * - extended_data:
     *   * if the data is planar audio with more than 8 channels, then this
     *     callback must allocate and fill extended_data to contain all pointers
     *     to all data planes. data[] must hold as many pointers as it can.
     *     extended_data must be allocated with av_malloc() and will be freed in
     *     av_frame_unref().
     *   * otherwise exended_data must point to data
     * - buf[] must contain one or more pointers to AVBufferRef structures. Each of
     *   the frame's data and extended_data pointers must be contained in these. That
     *   is, one AVBufferRef for each allocated chunk of memory, not necessarily one
     *   AVBufferRef per data[] entry. See: av_buffer_create(), av_buffer_alloc(),
     *   and av_buffer_ref().
     * - extended_buf and nb_extended_buf must be allocated with av_malloc() by
     *   this callback and filled with the extra buffers if there are more
     *   buffers than buf[] can hold. extended_buf will be freed in
     *   av_frame_unref().
     *
     * If AV_CODEC_CAP_DR1 is not set then get_buffer2() must call
     * avcodec_default_get_buffer2() instead of providing buffers allocated by
     * some other means.
     *
     * Each data plane must be aligned to the maximum required by the target
     * CPU.
     *
     * @see avcodec_default_get_buffer2()
     *
     * Video:
     *
     * If AV_GET_BUFFER_FLAG_REF is set in flags then the frame may be reused
     * (read and/or written to if it is writable) later by libavcodec.
     *
     * avcodec_align_dimensions2() should be used to find the required width and
     * height, as they normally need to be rounded up to the next multiple of 16.
     *
     * Some decoders do not support linesizes changing between frames.
     *
     * If frame multithreading is used and thread_safe_callbacks is set,
     * this callback may be called from a different thread, but not from more
     * than one at once. Does not need to be reentrant.
     *
     * @see avcodec_align_dimensions2()
     *
     * Audio:
     *
     * Decoders request a buffer of a particular size by setting
     * AVFrame.nb_samples prior to calling get_buffer2(). The decoder may,
     * however, utilize only part of the buffer by setting AVFrame.nb_samples
     * to a smaller value in the output frame.
     *
     * As a convenience, av_samples_get_buffer_size() and
     * av_samples_fill_arrays() in libavutil may be used by custom get_buffer2()
     * functions to find the required data size and to fill data pointers and
     * linesize. In AVFrame.linesize, only linesize[0] may be set for audio
     * since all planes must be the same size.
     *
     * @see av_samples_get_buffer_size(), av_samples_fill_arrays()
     *
     * - encoding: unused
     * - decoding: Set by libavcodec, user can override.
     */
    int (*get_buffer2)(struct AVCodecContext57 *s, AVFrame57 *frame, int flags);

    /**
     * If non-zero, the decoded audio and video frames returned from
     * avcodec_decode_video2() and avcodec_decode_audio4() are reference-counted
     * and are valid indefinitely. The caller must free them with
     * av_frame_unref() when they are not needed anymore.
     * Otherwise, the decoded frames must not be freed by the caller and are
     * only valid until the next decode call.
     *
     * - encoding: unused
     * - decoding: set by the caller before avcodec_open2().
     */
    int refcounted_frames;

    /* - encoding parameters */
    float qcompress;  ///< amount of qscale change between easy & hard scenes (0.0-1.0)
    float qblur;      ///< amount of qscale smoothing over time (0.0-1.0)

    /**
     * minimum quantizer
     * - encoding: Set by user.
     * - decoding: unused
     */
    int qmin;

    /**
     * maximum quantizer
     * - encoding: Set by user.
     * - decoding: unused
     */
    int qmax;

    /**
     * maximum quantizer difference between frames
     * - encoding: Set by user.
     * - decoding: unused
     */
    int max_qdiff;

#if FF_API_MPV_OPT
    /**
     * @deprecated use encoder private options instead
     */
    attribute_deprecated
    float rc_qsquish;

    attribute_deprecated
    float rc_qmod_amp;
    attribute_deprecated
    int rc_qmod_freq;
#endif

    /**
     * decoder bitstream buffer size
     * - encoding: Set by user.
     * - decoding: unused
     */
    int rc_buffer_size;

    /**
     * ratecontrol override, see RcOverride
     * - encoding: Allocated/set/freed by user.
     * - decoding: unused
     */
    int rc_override_count;
    RcOverride *rc_override;

#if FF_API_MPV_OPT
    /**
     * @deprecated use encoder private options instead
     */
    attribute_deprecated
    const char *rc_eq;
#endif

    /**
     * maximum bitrate
     * - encoding: Set by user.
     * - decoding: Set by user, may be overwritten by libavcodec.
     */
    int64_t rc_max_rate;

    /**
     * minimum bitrate
     * - encoding: Set by user.
     * - decoding: unused
     */
    int64_t rc_min_rate;

#if FF_API_MPV_OPT
    /**
     * @deprecated use encoder private options instead
     */
    attribute_deprecated
    float rc_buffer_aggressivity;

    attribute_deprecated
    float rc_initial_cplx;
#endif

    /**
     * Ratecontrol attempt to use, at maximum, <value> of what can be used without an underflow.
     * - encoding: Set by user.
     * - decoding: unused.
     */
    float rc_max_available_vbv_use;

    /**
     * Ratecontrol attempt to use, at least, <value> times the amount needed to prevent a vbv overflow.
     * - encoding: Set by user.
     * - decoding: unused.
     */
    float rc_min_vbv_overflow_use;

    /**
     * Number of bits which should be loaded into the rc buffer before decoding starts.
     * - encoding: Set by user.
     * - decoding: unused
     */
    int rc_initial_buffer_occupancy;

#if FF_API_CODER_TYPE
#define FF_CODER_TYPE_VLC_57       0
#define FF_CODER_TYPE_AC_57        1
#define FF_CODER_TYPE_RAW_57       2
#define FF_CODER_TYPE_RLE_57       3
#if FF_API_UNUSED_MEMBERS
#define FF_CODER_TYPE_DEFLATE_57   4
#endif /* FF_API_UNUSED_MEMBERS */
    /**
     * @deprecated use encoder private options instead
     */
    attribute_deprecated
    int coder_type;
#endif /* FF_API_CODER_TYPE */

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int context_model;
#endif

#if FF_API_MPV_OPT
    /**
     * @deprecated use encoder private options instead
     */
    attribute_deprecated
    int lmin;

    /**
     * @deprecated use encoder private options instead
     */
    attribute_deprecated
    int lmax;
#endif

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int frame_skip_threshold;

    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int frame_skip_factor;

    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int frame_skip_exp;

    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int frame_skip_cmp;
#endif /* FF_API_PRIVATE_OPT */

    /**
     * trellis RD quantization
     * - encoding: Set by user.
     * - decoding: unused
     */
    int trellis;

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int min_prediction_order;

    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int max_prediction_order;

    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int64_t timecode_frame_start;
#endif

#if FF_API_RTP_CALLBACK
    /**
     * @deprecated unused
     */
    /* The RTP callback: This function is called    */
    /* every time the encoder has a packet to send. */
    /* It depends on the encoder if the data starts */
    /* with a Start Code (it should). H.263 does.   */
    /* mb_nb contains the number of macroblocks     */
    /* encoded in the RTP payload.                  */
    attribute_deprecated
    void (*rtp_callback)(struct AVCodecContext57 *avctx, void *data, int size, int mb_nb);
#endif

#if FF_API_PRIVATE_OPT
    /** @deprecated use encoder private options instead */
    attribute_deprecated
    int rtp_payload_size;   /* The size of the RTP payload: the coder will  */
                            /* do its best to deliver a chunk with size     */
                            /* below rtp_payload_size, the chunk will start */
                            /* with a start code on some codecs like H.263. */
                            /* This doesn't take account of any particular  */
                            /* headers inside the transmitted RTP payload.  */
#endif

#if FF_API_STAT_BITS
    /* statistics, used for 2-pass encoding */
    attribute_deprecated
    int mv_bits;
    attribute_deprecated
    int header_bits;
    attribute_deprecated
    int i_tex_bits;
    attribute_deprecated
    int p_tex_bits;
    attribute_deprecated
    int i_count;
    attribute_deprecated
    int p_count;
    attribute_deprecated
    int skip_count;
    attribute_deprecated
    int misc_bits;

    /** @deprecated this field is unused */
    attribute_deprecated
    int frame_bits;
#endif

    /**
     * pass1 encoding statistics output buffer
     * - encoding: Set by libavcodec.
     * - decoding: unused
     */
    char *stats_out;

    /**
     * pass2 encoding statistics input buffer
     * Concatenated stuff from stats_out of pass1 should be placed here.
     * - encoding: Allocated/set/freed by user.
     * - decoding: unused
     */
    char *stats_in;

    /**
     * Work around bugs in encoders which sometimes cannot be detected automatically.
     * - encoding: Set by user
     * - decoding: Set by user
     */
    int workaround_bugs;
#define FF_BUG_AUTODETECT       1  ///< autodetection
#if FF_API_OLD_MSMPEG4
#define FF_BUG_OLD_MSMPEG4_57      2
#endif
#define FF_BUG_XVID_ILACE_57       4
#define FF_BUG_UMP4_57             8
#define FF_BUG_NO_PADDING_57       16
#define FF_BUG_AMV_57              32
#if FF_API_AC_VLC
#define FF_BUG_AC_VLC_57           0  ///< Will be removed, libavcodec can now handle these non-compliant files by default.
#endif
#define FF_BUG_QPEL_CHROMA_57      64
#define FF_BUG_STD_QPEL_57         128
#define FF_BUG_QPEL_CHROMA2_57     256
#define FF_BUG_DIRECT_BLOCKSIZE_57 512
#define FF_BUG_EDGE_57             1024
#define FF_BUG_HPEL_CHROMA_57      2048
#define FF_BUG_DC_CLIP_57          4096
#define FF_BUG_MS_57               8192 ///< Work around various bugs in Microsoft's broken decoders.
#define FF_BUG_TRUNCATED_57       16384

    /**
     * strictly follow the standard (MPEG4, ...).
     * - encoding: Set by user.
     * - decoding: Set by user.
     * Setting this to STRICT or higher means the encoder and decoder will
     * generally do stupid things, whereas setting it to unofficial or lower
     * will mean the encoder might produce output that is not supported by all
     * spec-compliant decoders. Decoders don't differentiate between normal,
     * unofficial and experimental (that is, they always try to decode things
     * when they can) unless they are explicitly asked to behave stupidly
     * (=strictly conform to the specs)
     */
    int strict_std_compliance;
#define FF_COMPLIANCE_VERY_STRICT_57   2 ///< Strictly conform to an older more strict version of the spec or reference software.
#define FF_COMPLIANCE_STRICT_57        1 ///< Strictly conform to all the things in the spec no matter what consequences.
#define FF_COMPLIANCE_NORMAL_57        0
#define FF_COMPLIANCE_UNOFFICIAL_57   -1 ///< Allow unofficial extensions
#define FF_COMPLIANCE_EXPERIMENTAL_57 -2 ///< Allow nonstandardized experimental things.

    /**
     * error concealment flags
     * - encoding: unused
     * - decoding: Set by user.
     */
    int error_concealment;
#define FF_EC_GUESS_MVS_57   1
#define FF_EC_DEBLOCK_57     2
#define FF_EC_FAVOR_INTER_57 256

    /**
     * debug
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    int debug;
#define FF_DEBUG_PICT_INFO_57   1
#define FF_DEBUG_RC_57          2
#define FF_DEBUG_BITSTREAM_57   4
#define FF_DEBUG_MB_TYPE_57     8
#define FF_DEBUG_QP_57          16
#if FF_API_DEBUG_MV
/**
 * @deprecated this option does nothing
 */
#define FF_DEBUG_MV_57          32
#endif
#define FF_DEBUG_DCT_COEFF_57   0x00000040
#define FF_DEBUG_SKIP_57        0x00000080
#define FF_DEBUG_STARTCODE_57   0x00000100
#if FF_API_UNUSED_MEMBERS
#define FF_DEBUG_PTS_57         0x00000200
#endif /* FF_API_UNUSED_MEMBERS */
#define FF_DEBUG_ER_57          0x00000400
#define FF_DEBUG_MMCO_57        0x00000800
#define FF_DEBUG_BUGS_57        0x00001000
#if FF_API_DEBUG_MV
#define FF_DEBUG_VIS_QP_57      0x00002000 ///< only access through AVOptions from outside libavcodec
#define FF_DEBUG_VIS_MB_TYPE_57 0x00004000 ///< only access through AVOptions from outside libavcodec
#endif
#define FF_DEBUG_BUFFERS_57     0x00008000
#define FF_DEBUG_THREADS_57     0x00010000
#define FF_DEBUG_GREEN_MD_57    0x00800000
#define FF_DEBUG_NOMC_57        0x01000000

#if FF_API_DEBUG_MV
    /**
     * debug
     * Code outside libavcodec should access this field using AVOptions
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    int debug_mv;
#define FF_DEBUG_VIS_MV_P_FOR_57  0x00000001 //visualize forward predicted MVs of P frames
#define FF_DEBUG_VIS_MV_B_FOR_57  0x00000002 //visualize forward predicted MVs of B frames
#define FF_DEBUG_VIS_MV_B_BACK_57 0x00000004 //visualize backward predicted MVs of B frames
#endif

    /**
     * Error recognition; may misdetect some more or less valid parts as errors.
     * - encoding: unused
     * - decoding: Set by user.
     */
    int err_recognition;

/**
 * Verify checksums embedded in the bitstream (could be of either encoded or
 * decoded data, depending on the codec) and print an error message on mismatch.
 * If AV_EF_EXPLODE is also set, a mismatching checksum will result in the
 * decoder returning an error.
 */
#define AV_EF_CRCCHECK_57  (1<<0)
#define AV_EF_BITSTREAM_57 (1<<1)          ///< detect bitstream specification deviations
#define AV_EF_BUFFER_57    (1<<2)          ///< detect improper bitstream length
#define AV_EF_EXPLODE_57   (1<<3)          ///< abort decoding on minor error detection

#define AV_EF_IGNORE_ERR_57 (1<<15)        ///< ignore errors and continue
#define AV_EF_CAREFUL_57    (1<<16)        ///< consider things that violate the spec, are fast to calculate and have not been seen in the wild as errors
#define AV_EF_COMPLIANT_57  (1<<17)        ///< consider all spec non compliances as errors
#define AV_EF_AGGRESSIVE (1<<18)        ///< consider things that a sane encoder should not do as an error


    /**
     * opaque 64bit number (generally a PTS) that will be reordered and
     * output in AVFrame.reordered_opaque
     * - encoding: unused
     * - decoding: Set by user.
     */
    int64_t reordered_opaque;

    /**
     * Hardware accelerator in use
     * - encoding: unused.
     * - decoding: Set by libavcodec
     */
    struct AVHWAccel *hwaccel;

    /**
     * Hardware accelerator context.
     * For some hardware accelerators, a global context needs to be
     * provided by the user. In that case, this holds display-dependent
     * data FFmpeg cannot instantiate itself. Please refer to the
     * FFmpeg HW accelerator documentation to know how to fill this
     * is. e.g. for VA API, this is a struct vaapi_context.
     * - encoding: unused
     * - decoding: Set by user
     */
    void *hwaccel_context;

    /**
     * error
     * - encoding: Set by libavcodec if flags & AV_CODEC_FLAG_PSNR.
     * - decoding: unused
     */
    uint64_t error[AV_NUM_DATA_POINTERS];

    /**
     * DCT algorithm, see FF_DCT_* below
     * - encoding: Set by user.
     * - decoding: unused
     */
    int dct_algo;
#define FF_DCT_AUTO_57    0
#define FF_DCT_FASTINT_57 1
#define FF_DCT_INT_57     2
#define FF_DCT_MMX_57     3
#define FF_DCT_ALTIVEC_57 5
#define FF_DCT_FAAN_57    6

    /**
     * IDCT algorithm, see FF_IDCT_* below.
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    int idct_algo;
#define FF_IDCT_AUTO_57          0
#define FF_IDCT_INT_57           1
#define FF_IDCT_SIMPLE_57        2
#define FF_IDCT_SIMPLEMMX_57     3
#define FF_IDCT_ARM_57           7
#define FF_IDCT_ALTIVEC_57       8
#if FF_API_ARCH_SH4
#define FF_IDCT_SH4_57           9
#endif
#define FF_IDCT_SIMPLEARM_57     10
#if FF_API_UNUSED_MEMBERS
#define FF_IDCT_IPP_57           13
#endif /* FF_API_UNUSED_MEMBERS */
#define FF_IDCT_XVID_57          14
#if FF_API_IDCT_XVIDMMX
#define FF_IDCT_XVIDMMX_57       14
#endif /* FF_API_IDCT_XVIDMMX */
#define FF_IDCT_SIMPLEARMV5TE_57 16
#define FF_IDCT_SIMPLEARMV6_57   17
#if FF_API_ARCH_SPARC
#define FF_IDCT_SIMPLEVIS_57     18
#endif
#define FF_IDCT_FAAN_57          20
#define FF_IDCT_SIMPLENEON_57    22
#if FF_API_ARCH_ALPHA
#define FF_IDCT_SIMPLEALPHA_57   23
#endif
#define FF_IDCT_SIMPLEAUTO_57    128

    /**
     * bits per sample/pixel from the demuxer (needed for huffyuv).
     * - encoding: Set by libavcodec.
     * - decoding: Set by user.
     */
     int bits_per_coded_sample;

    /**
     * Bits per sample/pixel of internal libavcodec pixel/sample format.
     * - encoding: set by user.
     * - decoding: set by libavcodec.
     */
    int bits_per_raw_sample;

#if FF_API_LOWRES
    /**
     * low resolution decoding, 1-> 1/2 size, 2->1/4 size
     * - encoding: unused
     * - decoding: Set by user.
     * Code outside libavcodec should access this field using:
     * av_codec_{get,set}_lowres(avctx)
     */
     int lowres;
#endif

#if FF_API_CODED_FRAME
    /**
     * the picture in the bitstream
     * - encoding: Set by libavcodec.
     * - decoding: unused
     *
     * @deprecated use the quality factor packet side data instead
     */
    attribute_deprecated AVFrame57 *coded_frame;
#endif

    /**
     * thread count
     * is used to decide how many independent tasks should be passed to execute()
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    int thread_count;

    /**
     * Which multithreading methods to use.
     * Use of FF_THREAD_FRAME will increase decoding delay by one frame per thread,
     * so clients which cannot provide future frames should not use it.
     *
     * - encoding: Set by user, otherwise the default is used.
     * - decoding: Set by user, otherwise the default is used.
     */
    int thread_type;
#define FF_THREAD_FRAME_57   1 ///< Decode more than one frame at once
#define FF_THREAD_SLICE_57   2 ///< Decode more than one part of a single frame at once

    /**
     * Which multithreading methods are in use by the codec.
     * - encoding: Set by libavcodec.
     * - decoding: Set by libavcodec.
     */
    int active_thread_type;

    /**
     * Set by the client if its custom get_buffer() callback can be called
     * synchronously from another thread, which allows faster multithreaded decoding.
     * draw_horiz_band() will be called from other threads regardless of this setting.
     * Ignored if the default get_buffer() is used.
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    int thread_safe_callbacks;

    /**
     * The codec may call this to execute several independent things.
     * It will return only after finishing all tasks.
     * The user may replace this with some multithreaded implementation,
     * the default implementation will execute the parts serially.
     * @param count the number of things to execute
     * - encoding: Set by libavcodec, user can override.
     * - decoding: Set by libavcodec, user can override.
     */
    int (*execute)(struct AVCodecContext57 *c, int (*func)(struct AVCodecContext57 *c2, void *arg), void *arg2, int *ret, int count, int size);

    /**
     * The codec may call this to execute several independent things.
     * It will return only after finishing all tasks.
     * The user may replace this with some multithreaded implementation,
     * the default implementation will execute the parts serially.
     * Also see avcodec_thread_init and e.g. the --enable-pthread configure option.
     * @param c context passed also to func
     * @param count the number of things to execute
     * @param arg2 argument passed unchanged to func
     * @param ret return values of executed functions, must have space for "count" values. May be NULL.
     * @param func function that will be called count times, with jobnr from 0 to count-1.
     *             threadnr will be in the range 0 to c->thread_count-1 < MAX_THREADS and so that no
     *             two instances of func executing at the same time will have the same threadnr.
     * @return always 0 currently, but code should handle a future improvement where when any call to func
     *         returns < 0 no further calls to func may be done and < 0 is returned.
     * - encoding: Set by libavcodec, user can override.
     * - decoding: Set by libavcodec, user can override.
     */
    int (*execute2)(struct AVCodecContext57 *c, int (*func)(struct AVCodecContext57 *c2, void *arg, int jobnr, int threadnr), void *arg2, int *ret, int count);

    /**
     * noise vs. sse weight for the nsse comparison function
     * - encoding: Set by user.
     * - decoding: unused
     */
     int nsse_weight;

    /**
     * profile
     * - encoding: Set by user.
     * - decoding: Set by libavcodec.
     */
     int profile;
#define FF_PROFILE_UNKNOWN_57 -99
#define FF_PROFILE_RESERVED_57 -100

#define FF_PROFILE_AAC_MAIN_57 0
#define FF_PROFILE_AAC_LOW_57  1
#define FF_PROFILE_AAC_SSR_57  2
#define FF_PROFILE_AAC_LTP_57  3
#define FF_PROFILE_AAC_HE_57   4
#define FF_PROFILE_AAC_HE_V2_57 28
#define FF_PROFILE_AAC_LD_57   22
#define FF_PROFILE_AAC_ELD_57  38
#define FF_PROFILE_MPEG2_AAC_LOW_57 128
#define FF_PROFILE_MPEG2_AAC_HE_57  131

#define FF_PROFILE_DTS_57         20
#define FF_PROFILE_DTS_ES_57      30
#define FF_PROFILE_DTS_96_24_57   40
#define FF_PROFILE_DTS_HD_HRA_57  50
#define FF_PROFILE_DTS_HD_MA_57   60
#define FF_PROFILE_DTS_EXPRESS_57 70

#define FF_PROFILE_MPEG2_422_57    0
#define FF_PROFILE_MPEG2_HIGH_57   1
#define FF_PROFILE_MPEG2_SS_57     2
#define FF_PROFILE_MPEG2_SNR_SCALABLE_57  3
#define FF_PROFILE_MPEG2_MAIN_57   4
#define FF_PROFILE_MPEG2_SIMPLE_57 5

#define FF_PROFILE_H264_CONSTRAINED_57  (1<<9)  // 8+1; constraint_set1_flag
#define FF_PROFILE_H264_INTRA_57        (1<<11) // 8+3; constraint_set3_flag

#define FF_PROFILE_H264_BASELINE_57             66
#define FF_PROFILE_H264_CONSTRAINED_BASELINE_57 (66|FF_PROFILE_H264_CONSTRAINED)
#define FF_PROFILE_H264_MAIN_57                 77
#define FF_PROFILE_H264_EXTENDED_57             88
#define FF_PROFILE_H264_HIGH_57                 100
#define FF_PROFILE_H264_HIGH_10_57              110
#define FF_PROFILE_H264_HIGH_10_INTRA_57        (110|FF_PROFILE_H264_INTRA)
#define FF_PROFILE_H264_HIGH_422_57             122
#define FF_PROFILE_H264_HIGH_422_INTRA_57       (122|FF_PROFILE_H264_INTRA)
#define FF_PROFILE_H264_HIGH_444_57             144
#define FF_PROFILE_H264_HIGH_444_PREDICTIVE_57  244
#define FF_PROFILE_H264_HIGH_444_INTRA_57       (244|FF_PROFILE_H264_INTRA)
#define FF_PROFILE_H264_CAVLC_444_57            44

#define FF_PROFILE_VC1_SIMPLE_57   0
#define FF_PROFILE_VC1_MAIN_57     1
#define FF_PROFILE_VC1_COMPLEX_57  2
#define FF_PROFILE_VC1_ADVANCED_57 3

#define FF_PROFILE_MPEG4_SIMPLE_57                     0
#define FF_PROFILE_MPEG4_SIMPLE_SCALABLE_57            1
#define FF_PROFILE_MPEG4_CORE_57                       2
#define FF_PROFILE_MPEG4_MAIN_57                       3
#define FF_PROFILE_MPEG4_N_BIT_57                      4
#define FF_PROFILE_MPEG4_SCALABLE_TEXTURE_57           5
#define FF_PROFILE_MPEG4_SIMPLE_FACE_ANIMATION_57      6
#define FF_PROFILE_MPEG4_BASIC_ANIMATED_TEXTURE_57     7
#define FF_PROFILE_MPEG4_HYBRID_57                     8
#define FF_PROFILE_MPEG4_ADVANCED_REAL_TIME_57         9
#define FF_PROFILE_MPEG4_CORE_SCALABLE_57             10
#define FF_PROFILE_MPEG4_ADVANCED_CODING_57           11
#define FF_PROFILE_MPEG4_ADVANCED_CORE_57             12
#define FF_PROFILE_MPEG4_ADVANCED_SCALABLE_TEXTURE_57 13
#define FF_PROFILE_MPEG4_SIMPLE_STUDIO_57             14
#define FF_PROFILE_MPEG4_ADVANCED_SIMPLE_57           15

#define FF_PROFILE_JPEG2000_CSTREAM_RESTRICTION_0_57   0
#define FF_PROFILE_JPEG2000_CSTREAM_RESTRICTION_1_57   1
#define FF_PROFILE_JPEG2000_CSTREAM_NO_RESTRICTION_57  2
#define FF_PROFILE_JPEG2000_DCINEMA_2K_57              3
#define FF_PROFILE_JPEG2000_DCINEMA_4K_57              4

#define FF_PROFILE_VP9_0_57                            0
#define FF_PROFILE_VP9_1_57                            1
#define FF_PROFILE_VP9_2_57                            2
#define FF_PROFILE_VP9_3_57                            3

#define FF_PROFILE_HEVC_MAIN_57                        1
#define FF_PROFILE_HEVC_MAIN_10_57                     2
#define FF_PROFILE_HEVC_MAIN_STILL_PICTURE_57          3
#define FF_PROFILE_HEVC_REXT_57                        4

    /**
     * level
     * - encoding: Set by user.
     * - decoding: Set by libavcodec.
     */
     int level;
#define FF_LEVEL_UNKNOWN_57 -99

    /**
     * Skip loop filtering for selected frames.
     * - encoding: unused
     * - decoding: Set by user.
     */
    enum AVDiscard skip_loop_filter;

    /**
     * Skip IDCT/dequantization for selected frames.
     * - encoding: unused
     * - decoding: Set by user.
     */
    enum AVDiscard skip_idct;

    /**
     * Skip decoding for selected frames.
     * - encoding: unused
     * - decoding: Set by user.
     */
    enum AVDiscard skip_frame;

    /**
     * Header containing style information for text subtitles.
     * For SUBTITLE_ASS subtitle type, it should contain the whole ASS
     * [Script Info] and [V4+ Styles] section, plus the [Events] line and
     * the Format line following. It shouldn't include any Dialogue line.
     * - encoding: Set/allocated/freed by user (before avcodec_open2())
     * - decoding: Set/allocated/freed by libavcodec (by avcodec_open2())
     */
    uint8_t *subtitle_header;
    int subtitle_header_size;

#if FF_API_ERROR_RATE
    /**
     * @deprecated use the 'error_rate' private AVOption of the mpegvideo
     * encoders
     */
    attribute_deprecated
    int error_rate;
#endif

#if FF_API_VBV_DELAY
    /**
     * VBV delay coded in the last frame (in periods of a 27 MHz clock).
     * Used for compliant TS muxing.
     * - encoding: Set by libavcodec.
     * - decoding: unused.
     * @deprecated this value is now exported as a part of
     * AV_PKT_DATA_CPB_PROPERTIES packet side data
     */
    attribute_deprecated
    uint64_t vbv_delay;
#endif

#if FF_API_SIDEDATA_ONLY_PKT
    /**
     * Encoding only and set by default. Allow encoders to output packets
     * that do not contain any encoded data, only side data.
     *
     * Some encoders need to output such packets, e.g. to update some stream
     * parameters at the end of encoding.
     *
     * @deprecated this field disables the default behaviour and
     *             it is kept only for compatibility.
     */
    attribute_deprecated
    int side_data_only_packets;
#endif

    /**
     * Audio only. The number of "priming" samples (padding) inserted by the
     * encoder at the beginning of the audio. I.e. this number of leading
     * decoded samples must be discarded by the caller to get the original audio
     * without leading padding.
     *
     * - decoding: unused
     * - encoding: Set by libavcodec. The timestamps on the output packets are
     *             adjusted by the encoder so that they always refer to the
     *             first sample of the data actually contained in the packet,
     *             including any added padding.  E.g. if the timebase is
     *             1/samplerate and the timestamp of the first input sample is
     *             0, the timestamp of the first output packet will be
     *             -initial_padding.
     */
    int initial_padding;

    /**
     * - decoding: For codecs that store a framerate value in the compressed
     *             bitstream, the decoder may export it here. { 0, 1} when
     *             unknown.
     * - encoding: unused
     */
    AVRational framerate;

    /**
     * Nominal unaccelerated pixel format, see AV_PIX_FMT_xxx.
     * - encoding: unused.
     * - decoding: Set by libavcodec before calling get_format()
     */
    AVPixelFormat57 sw_pix_fmt;

    /**
     * Timebase in which pkt_dts/pts and AVPacket.dts/pts are.
     * Code outside libavcodec should access this field using:
     * av_codec_{get,set}_pkt_timebase(avctx)
     * - encoding unused.
     * - decoding set by user.
     */
    AVRational pkt_timebase;

    /**
     * AVCodecDescriptor
     * Code outside libavcodec should access this field using:
     * av_codec_{get,set}_codec_descriptor(avctx)
     * - encoding: unused.
     * - decoding: set by libavcodec.
     */
    const AVCodecDescriptor *codec_descriptor;

#if !FF_API_LOWRES
    /**
     * low resolution decoding, 1-> 1/2 size, 2->1/4 size
     * - encoding: unused
     * - decoding: Set by user.
     * Code outside libavcodec should access this field using:
     * av_codec_{get,set}_lowres(avctx)
     */
     int lowres;
#endif

    /**
     * Current statistics for PTS correction.
     * - decoding: maintained and used by libavcodec, not intended to be used by user apps
     * - encoding: unused
     */
    int64_t pts_correction_num_faulty_pts; /// Number of incorrect PTS values so far
    int64_t pts_correction_num_faulty_dts; /// Number of incorrect DTS values so far
    int64_t pts_correction_last_pts;       /// PTS of the last frame
    int64_t pts_correction_last_dts;       /// DTS of the last frame

    /**
     * Character encoding of the input subtitles file.
     * - decoding: set by user
     * - encoding: unused
     */
    char *sub_charenc;

    /**
     * Subtitles character encoding mode. Formats or codecs might be adjusting
     * this setting (if they are doing the conversion themselves for instance).
     * - decoding: set by libavcodec
     * - encoding: unused
     */
    int sub_charenc_mode;
#define FF_SUB_CHARENC_MODE_DO_NOTHING  -1  ///< do nothing (demuxer outputs a stream supposed to be already in UTF-8, or the codec is bitmap for instance)
#define FF_SUB_CHARENC_MODE_AUTOMATIC    0  ///< libavcodec will select the mode itself
#define FF_SUB_CHARENC_MODE_PRE_DECODER  1  ///< the AVPacket data needs to be recoded to UTF-8 before being fed to the decoder, requires iconv

    /**
     * Skip processing alpha if supported by codec.
     * Note that if the format uses pre-multiplied alpha (common with VP6,
     * and recommended due to better video quality/compression)
     * the image will look as if alpha-blended onto a black background.
     * However for formats that do not use pre-multiplied alpha
     * there might be serious artefacts (though e.g. libswscale currently
     * assumes pre-multiplied alpha anyway).
     * Code outside libavcodec should access this field using AVOptions
     *
     * - decoding: set by user
     * - encoding: unused
     */
    int skip_alpha;

    /**
     * Number of samples to skip after a discontinuity
     * - decoding: unused
     * - encoding: set by libavcodec
     */
    int seek_preroll;

#if !FF_API_DEBUG_MV
    /**
     * debug motion vectors
     * Code outside libavcodec should access this field using AVOptions
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    int debug_mv;
#define FF_DEBUG_VIS_MV_P_FOR_57  0x00000001 //visualize forward predicted MVs of P frames
#define FF_DEBUG_VIS_MV_B_FOR_57  0x00000002 //visualize forward predicted MVs of B frames
#define FF_DEBUG_VIS_MV_B_BACK_57 0x00000004 //visualize backward predicted MVs of B frames
#endif

    /**
     * custom intra quantization matrix
     * Code outside libavcodec should access this field using av_codec_g/set_chroma_intra_matrix()
     * - encoding: Set by user, can be NULL.
     * - decoding: unused.
     */
    uint16_t *chroma_intra_matrix;

    /**
     * dump format separator.
     * can be ", " or "\n      " or anything else
     * Code outside libavcodec should access this field using AVOptions
     * (NO direct access).
     * - encoding: Set by user.
     * - decoding: Set by user.
     */
    uint8_t *dump_separator;

    /**
     * ',' separated list of allowed decoders.
     * If NULL then all are allowed
     * - encoding: unused
     * - decoding: set by user through AVOPtions (NO direct access)
     */
    char *codec_whitelist;

    /*
     * Properties of the stream that gets decoded
     * To be accessed through av_codec_get_properties() (NO direct access)
     * - encoding: unused
     * - decoding: set by libavcodec
     */
    unsigned properties;
#define FF_CODEC_PROPERTY_LOSSLESS        0x00000001
#define FF_CODEC_PROPERTY_CLOSED_CAPTIONS 0x00000002

    /**
     * Additional data associated with the entire coded stream.
     *
     * - decoding: unused
     * - encoding: may be set by libavcodec after avcodec_open2().
     */
    AVPacketSideData57 *coded_side_data;
    int            nb_coded_side_data;

} AVCodecContext57;

#endif // #if LIBAVCODEC_VERSION_MAJOR == 57

#endif // #ifndef AVCODEC_AVCODEC_57_H
